@@ -1,17 +1,20 @@
 use core::fmt;
 
 /// Encoding errors.
-#[non_exhaustive]
 #[derive(Debug, Clone)]
+#[non_exhaustive]
 pub enum Error<W> {
     /// Error writing bytes to a `Write` impl.
-    Write(W)
+    Write(W),
+    /// Generic error message.
+    Message(&'static str)
 }
 
 impl<W: fmt::Display> fmt::Display for Error<W> {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         match self {
-            Error::Write(e) => write!(f, "write error: {}", e),
+            Error::Write(e)   => write!(f, "write error: {}", e),
+            Error::Message(m) => write!(f, "{}", m)
         }
     }
 }
@@ -20,7 +23,8 @@ impl<W: fmt::Display> fmt::Display for Error<W> {
 impl<W: std::error::Error + 'static> std::error::Error for Error<W> {
     fn source(&self) -> Option<&(dyn std::error::Error + 'static)> {
         match self {
-            Error::Write(e) => Some(e)
+            Error::Write(e)   => Some(e),
+            Error::Message(_) => None
         }
     }
 }
